/* 
 * mme_jvm_api.h
 *
 * 
 *
 */
 /*
 *
 * Copyright (C) 2005       Yoav Zach <yoav.zach@intel.com>
 *
 */
 /*
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * or view it at http://www.gnu.org/licenses/lgpl.html
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
#ifndef _MME_JVM_API_H
#define _MME_JVM_API_H

#include <signal.h>
#include <stdbool.h>
#include <stdint.h>
#include <link.h>
#include <jni.h>

/* data types indentifiers */
#define MME_JAVA_DATA_TYPE_LONG		10         // primitives
#define MME_JAVA_DATA_TYPE_DOUBLE	9
#define MME_JAVA_DATA_TYPE_INT		8
#define MME_JAVA_DATA_TYPE_FLOAT	7
#define MME_JAVA_DATA_TYPE_SHORT	6
#define MME_JAVA_DATA_TYPE_CHAR		5
#define MME_JAVA_DATA_TYPE_BYTE		4
#define MME_JAVA_DATA_TYPE_BOOLEAN	3
#define MME_JAVA_DATA_TYPE_VOID		2
#define MME_JAVA_DATA_TYPE_OBJECT	1             // object
#define MAX_PARAM_STR_LEN		256


/* return values */
typedef enum {MME_SUCCESS = 0, MME_FAILURE = 1} mme_retval;
typedef enum {MME_SIGNAL_IA32_EXCEPTION = -1, MME_SIGNAL_NOT_HANDLED = 0, MME_SIGNAL_HANDLED = 1} mme_sighand_retval;

/* version */
#define MTL_VER_MAJOR "0"
#define MTL_VER_MINOR "1"
#define MME_MTL_VER_STR_LEN 64

/* initialization flags */
#define MME_MTL_SIG_HANDLE 1
#define MME_MTL_THREAD_INIT_EAGER 2

struct dl_phdr_info64 {
        Elf64_Addr dlpi_addr;
        char *dlpi_name;
        Elf64_Phdr *dlpi_phdr;
        Elf64_Half dlpi_phnum;
};
typedef struct {
	struct {
		void *cb;
		void *data;
	} jvm_sem_data;
	void **eip32;
	int   *is_ia32_native_exception;
} mme_sem_data;

/* jvm callback functions */
struct _mme_java_callback_tag {
	uint64_t (*get_primitive_array_element_size)(void * env, void * array);
	jint (*register_natives_32)(JNIEnv *env, jclass clazz, const JNINativeMethod *methods, jint nMethods);
	jint (*unregister_natives_32)(JNIEnv *env, jclass clazz);
	void (*run_jvm_handler)(void *siginfo);
};

/* api function prototypes */

/* library handling */

/******************************************************************************
 * Name: mme_version
 * Description: MME version checking
 * Syntax :
 * 	1.	Input parameters: None
 * 	2.	Output parameters:
 * 		i.	"version": Will hold version ID of the supported MME API.
 * 	3.	Return value: the constant MME_MTL_SUCCESS will be returned on
 *              success, and MME_MTL_FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_version (char version[MME_MTL_VER_STR_LEN]);

/******************************************************************************
 * Name : mme_init
 * Description : Initialization of MTL.
 * Syntax :
 * 1.	Input parameters:
 * 	i.	"mode": Flags that are used to determine different aspects of
 *                      work. The flags can be or'ed to define several aspects
 *                      with the same flags word. Currently, the only allowed
 *                      flags are:
 * 		a.	MME_MTL_SIG_HANDLE : determine whether the IA-32
 *                      compatibility layer (IA32EL) can register its own signal
 *                      handlers 
 * 		b.	MME_MTL_THREAD_INIT_EAGER : determines the mode of work
 *                      with threads. When this flag is set, the Java world will
 *                      inform the MTL on each and every thread creation, at the
 *                      point of creation (see section ?7.5.1).
 * 	ii.	"callbacks": The Java world can register callback functions to be
 *              used by the MTL.
 * 	iii.	"tls_offset": The Java world is responsible for reserving 8 byte
 *              slot at a fixed offset from r13 for all threads. This argument
 *              holds this offset.
 * 2.	Output parameters: None
 * 3.	Return value: the constant MME_SUCCESS will be returned on success, and
 *      MME__FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_init (uint64_t flags, struct _mme_java_callback_tag * callbacks, int64_t tls_offset);

/******************************************************************************
 * Name : mme_unload
 * Description : Destructor for btlib.
 * Syntax :
 * 	1.	Input parameters:
 * 		i.	"reason": Specifies the reason for unloading MTL.
 * 	2.	Output parameters: None
 * 	3.	Return value: the constant MME_SUCCESS will be returned on
 *              success, and MME__FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_unload (int32_t reason);

/* 32 bit dlfcn functions */

/******************************************************************************
 * Name : mme_dlopen
 * Description : Open an IA-32 library.
 * Syntax : See the man page for dlopen(3).
 ******************************************************************************/
extern void * mme_dlopen (const char * filename, int32_t flag);
/******************************************************************************
 * Name : mme_dlerror
 * Description : Return error code for the last mme_ia32_* operation.
 * Syntax : See the man page for dlerror(3).
 ******************************************************************************/
extern char * mme_dlerror (void);
/******************************************************************************
 * Name : mme_dlsym
 * Description : Lookup a symbol in an IA-32 library
 * Syntax : See the man page for dlsym(3).
 ******************************************************************************/
extern void * mme_dlsym (void * handle, const char * symbol);
/******************************************************************************
 * Name : mme_dlclose
 * Description : Close an IA-32 library.
 * Syntax : See the man page for dlclose(3).
 ******************************************************************************/
extern int32_t mme_dlclose (void * handle);

/* downcalls */

/******************************************************************************
 * Name : mme_downcall
 * Description : Issue a downcall.
 * Syntax :
 * 	1.	Input parameters:
 * 		i.	"return_type": Specifies the return type of the downcall
 *                      function.
 * 		ii.	"addr": Address of the function as was returned by
 *                      "mme_dlopen"
 * 			a. "param_list": A string that encodes the
 *                         parameters that are passed to the native function.
 *                         N'th character in the string represents the type of
 *                         N'th parameter passed to the native function.
 * 		iii.	"?": the parameters to the native function
 * 	2.	Output parameters: None
 * 	3.	Return value: the return value of the native function
 ******************************************************************************/
extern uint64_t mme_downcall (int64_t return_type, void *addr, char *param_list, ...);
extern uint64_t mme_downcallV (int64_t return_type, void *addr, char *param_list, va_list params);

/* signal handling */

/******************************************************************************
 * Name : mme_signal_handler
 * Description : Signal handler for MTL.
 * Syntax :
 * 	1.	Input parameters: See the man page for sigaction(2).
 * 	2.	Output parameters: None
 * 	3.	Return value: In case the signal was recognized and processed by
 *              IA32EL return value will be MME_SIGNAL_HANDLED. Otherwise the
 *              function will return MME_SIGNAL_NOT_HANDLED.
 ******************************************************************************/
extern mme_sighand_retval mme_signal_handler (int32_t signo, struct siginfo * si, struct sigcontext * sc);

/******************************************************************************
 * Name : mme_change_sighand
 * Description : Informs MTL about signal handling changes in the Java world
 * Syntax
 * 	1.	Input parameters: 
 * 		i.	"signo": The signal for which the handler changed
 * 		ii.	"sc": The details of the new handler
 * 	2.	Output parameters: None
 * 	3.	Return value: MME_SUCCESS on success, MME_FAILURE on failure.
 ******************************************************************************/
extern mme_retval mme_change_sighand (int32_t signo, struct sigaction * sc);

/* thread handling */

typedef enum {
	MME_THREAD_INIT_FAIL = -1,
	MME_THREAD_INIT_CHILD,
	MME_THREAD_INIT_PARENT,
} mme_thread_init_flag;



/******************************************************************************
 * Name : mme_thread_init
 * Description : Inform of thread creation.
 * Syntax
 * 	1.	Input parameters: None
 * 	2.	Output parameters: 
 *		i.	"clone_params": Will hold the parameter ia32el needs to
 *			pass to the child thread as soon as it is ready
 * 	3.	Return value: the constant MME_SUCCESS will be returned on
 *              success, and MME_FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_thread_init (void **clone_params);

/******************************************************************************
 * Name : mme_complete_thread_init
 * Description : Complete thread creation.
 * Syntax
 * 	1.	Input parameters:
 *		i.	"flag": Indicate in which context the function is called
 *			a.	MME_THREAD_INIT_FAIL : Creation of the new thread
 *				failed.
 *			b.	MME_THREAD_INIT_PARENT : The function is called
 *				in the context of the parent thread
 *			c.	MME_THREAD_INIT_CHILD : The function is called
 *				in the context of the child thread
 *		ii.	"clone_params": The parameter that ia32el passes to
 *			the child thread, as was received in mme_thread_init
 * 	2.	Output parameters: None
 * 	3.	Return value: the constant MME_SUCCESS will be returned on
 *              success, and MME_FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_complete_thread_init (mme_thread_init_flag flag, void *clone_params);
/******************************************************************************
 * Name : mme_thread_term
 * Description : Inform of IA-32 thread termination.
 * Syntax
 * 	1.	Input parameters: None
 * 	2.	Output parameters: None
 * 	3.	Return value: the constant MME_SUCCESS will be returned on
 *              success, and MME_FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_thread_term (void);

typedef int32_t (*phdr_iter_callback) (struct dl_phdr_info64 *, size_t, void *);
extern mme_retval mme_dl_iterate_phdr (phdr_iter_callback jvm_callback, void * data);

/******************************************************************************
 * Name : mme_dump_foreign_context
 * Description : Dump IA-32 context upon native exception
 * Syntax
 * 	1.	Input parameters: None
 * 	2.	Output parameters: None
 * 	3.	Return value: the constant MME_SUCCESS will be returned on
 *              success, and MME_FAILURE will be returned on failure.
 ******************************************************************************/
extern mme_retval mme_dump_foreign_context (FILE *fp);

#endif // _MME_JVM_API_H
